/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.xbean.classpath;

/**
 * @version $Revision$ $Date$
 */

import junit.framework.TestCase;

public class SystemClassPathTest extends TestCase {

    public void testAddJarToPath() throws Exception {
        SystemClassPath systemClassPath = new SystemClassPath();

        ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();

        try {
            systemClassLoader.loadClass("groovy.lang.GroovyShell");
            fail("Class already exists");
        } catch (ClassNotFoundException e) {
            // this should fail
        }


//        URL groovyJar = new URL("http://www.ibiblio.org/maven/groovy/jars/groovy-SNAPSHOT.jar");
//        systemClassPath.addJarToPath(groovyJar);
//
//        try {
//            systemClassLoader.loadClass("groovy.lang.GroovyShell");
//        } catch (ClassNotFoundException e) {
//            // this should fail pass
//            fail("Class already exists");
//        }
    }
}
