/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */


package org.apache.xbean.blueprint.context.impl;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;
import org.apache.xbean.blueprint.cm.JexlExpressionParser;

/**
 * @version $Rev: 907189 $ $Date: 2010-02-06 09:01:43 +0100 (Sat, 06 Feb 2010) $
 */
public class JexlTest extends TestCase {
    
    public void testJexl() throws Exception {
        Map<String, Object> vars = new HashMap<String, Object> ();
        vars.put("foo", 1);
        vars.put("bar", 2);
        JexlExpressionParser parser = new JexlExpressionParser(vars);
        assertEquals(3, parser.evaluate("${foo + bar}"));

        vars.put("ServerHostname", "localhost");
        vars.put("ActiveMQPort", 63636);
        vars.put("PortOffset", 10);
        assertEquals("tcp://localhost:63646", parser.evaluate("tcp://${ServerHostname}:${ActiveMQPort + PortOffset}"));
    }
}
