/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.xbean.spring.context.impl;

import java.lang.reflect.Constructor;
import java.util.List;

import org.apache.xbean.spring.context.SpringApplicationContext;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;

public class XBeanHelper {

    public static XmlBeanDefinitionReader createBeanDefinitionReader(
                    SpringApplicationContext applicationContext,
                    BeanDefinitionRegistry registry,
                    List xmlPreprocessors) {
        
        String version = "2.0";
        
        try {
            Class spring20Clazz = Class.forName("org.springframework.core.AttributeAccessorSupport");
            version = "2.0";
        } catch(ClassNotFoundException e) {
            version = "1.2.8";
        }
        
        String className = "org.apache.xbean.spring.context.v" + version.charAt(0) + ".XBeanXmlBeanDefinitionReader";
        try {
            Class cl = Class.forName(className);
            Constructor cstr = cl.getConstructor(new Class[] { SpringApplicationContext.class, BeanDefinitionRegistry.class, List.class });
            return (XmlBeanDefinitionReader) cstr.newInstance(new Object[] { applicationContext, registry, xmlPreprocessors });
        } catch (Exception e) {
            throw (IllegalStateException) new IllegalStateException("Could not find valid implementation for: " + version).initCause(e);
        }
    }
}
